"""Test with fixtures"""


def test_can_access_local_fixture(testdir):
    testdir.makepyfile(
        """
        import pytest

        def describe_something():
            @pytest.fixture
            def thing():
                return 42

            def thing_is_42(thing):
                assert thing == 42
        """)

    result = testdir.runpytest()
    result.assert_outcomes(passed=1)


def test_can_access_fixture_from_nested_scope(testdir):
    testdir.makepyfile(
        """
        import pytest

        def describe_something():
            @pytest.fixture
            def thing():
                return 42

            def describe_a_nested_scope():
                def thing_is_42(thing):
                    assert thing == 42
        """)

    result = testdir.runpytest()
    result.assert_outcomes(passed=1)


def test_local_fixture_overrides(testdir):
    testdir.makepyfile(
        """
        import pytest

        @pytest.fixture
        def thing():
            return 12

        def describe_something():
            def describe_a_nested_scope():
                @pytest.fixture
                def thing():
                    return 42

                def thing_is_42(thing):
                    assert thing == 42

            def thing_is_12(thing):
                assert thing == 12
        """)

    result = testdir.runpytest()
    result.assert_outcomes(passed=2)



"""Test verbose output"""


def test_verbose_output(testdir):
    testdir.makepyfile(
        """
        def describe_something():
            def describe_nested_ok():
                def passes():
                    assert True
            def describe_nested_bad():
                def fails():
                    assert False
        """
    )

    result = testdir.runpytest("-v")

    result.assert_outcomes(passed=1, failed=1)

    output = [
        ' '.join(line.split('::', 2)[2].split())
        for line in result.outlines
        if line.startswith('test_verbose_output.py::describe_something::')
    ]

    assert output == [
        "describe_nested_ok::passes PASSED [ 50%]",
        "describe_nested_bad::fails FAILED [100%]",
    ]



"""Support for shared behaviors"""

__all__ = ["behaves_like"]


def behaves_like(*behavior_funcs):
    """Decorator for shared behaviors."""

    def decorator(func):
        try:
            func._behaves_like.extend(behavior_funcs)
        except AttributeError:
            func._behaves_like = behavior_funcs[:]
        return func

    return decorator



"""Test simple execution"""


def test_can_pass(testdir):
    testdir.makepyfile(
        """
        def describe_something():
            def passes():
                assert True
            def describe_nested():
                def passes_too():
                    assert True
        """)

    result = testdir.runpytest()
    result.assert_outcomes(passed=2)


def test_can_fail(testdir):
    testdir.makepyfile(
        """
        def describe_something():
            def fails():
                assert False
            def describe_nested():
                def fails_too():
                    assert False
        """)

    result = testdir.runpytest()
    result.assert_outcomes(failed=2)


def test_can_fail_and_pass(testdir):
    testdir.makepyfile(
        """
        def describe_something():
            def describe_nested_ok():
                def passes():
                    assert True
            def describe_nested_bad():
                def fails():
                    assert False
        """)

    result = testdir.runpytest()
    result.assert_outcomes(passed=1, failed=1)



"""Test that classes are ignored"""


def test_skip_classes(testdir):
    testdir.makepyfile(
        """
        def describe_something():
            def fn():
                assert True
            class cls:
                def __call__(self):
                    assert True
        """)

    result = testdir.runpytest()
    result.assert_outcomes(passed=1)



"""Test collection of test functions"""

from textwrap import dedent


def test_collect_only(testdir):
    testdir.makepyfile(
        """
        def describe_something():
            def is_foo():
                pass
            def can_bar():
                pass
            def _not_a_test():
                pass
        def describe_something_else():
            def describe_nested():
                def a_test():
                    pass
        def foo_not_collected():
            pass
        def test_something():
            pass
        """)

    result = testdir.runpytest('--collectonly')
    result.assert_outcomes()

    output = '\n'.join(line.lstrip() for line in result.outlines)
    assert "collected 4 items" in output
    assert dedent("""
        <Module test_collect_only.py>
        <DescribeBlock 'describe_something'>
        <Function is_foo>
        <Function can_bar>
        <DescribeBlock 'describe_something_else'>
        <DescribeBlock 'describe_nested'>
        <Function a_test>
        <Function test_something>
        """) in output


def test_describe_evaluated_once(testdir):
    testdir.makepyfile(
        """
        count = 0
        def describe_is_evaluated_only_once():
            global count
            count += 1
            def one():
                assert count == 1
            def two():
                assert count == 1
            def describe_nested():
                def three():
                    assert count == 1
    """)

    result = testdir.runpytest('-v')
    result.assert_outcomes(passed=3)



"""Test custom prefixes"""

from textwrap import dedent


def test_collect_custom_prefix(testdir):
    testdir.makeini(
        """
        [pytest]
        describe_prefixes = foo bar
        """)

    testdir.makepyfile(
        """
        def foo_scope():
            def bar_context():
                def passes():
                    pass
        """)

    result = testdir.runpytest('--collectonly')
    result.assert_outcomes()

    output = '\n'.join(line.lstrip() for line in result.outlines if line)
    assert "collected 1 item" in output
    assert dedent("""
        <Module test_collect_custom_prefix.py>
        <DescribeBlock 'foo_scope'>
        <DescribeBlock 'bar_context'>
        <Function passes>
        """) in output
